==> proginoskes-0.1.2.tgz: package/lib/proginoskes.js <==
/*!
 * proginoskes
 * Copyright(c) 2014-2015 Jason Gerfen <jason.gerfen@gmail.com>
 * License: MIT
 */

var version = 'v0.1.2'
  , fs = require('fs')
  , os = require('os')
  , ssh = require('ssh2')
  , async = require('async')
  , merge = require('deepmerge')
  , conn = new ssh()
  , proginoskes = function(options, fn) {

  'use strict';

  /**
   * @object defaults
   * @abstract Default set of options
   *
   * @param Object} global - Global object for proginoskes
   * @param {Array} hosts - Array of configured streams
   */
  var defaults = {
    global: {
      threshold: os.cpus().length * 2,
      port: 22,
      logs: [
        '/var/log/ufw.log'
      ],
      ignore: {
        destination: [
          'localhost',
          '127.0.0.1'
        ],
        source: [
          'localhost',
          '127.0.0.1'
        ],
        ports: [
          53
        ]
      }
    },
    hosts: []
  };

  /**
   * @method config
   * @scope private
   * @abstract Configuration object
   */
  var config = config || {

    /**
     * @function init
     * @scope private
     * @abstract Creates new host object(s) from configuration
     *
     * @param {Object} obj Merged options
     *
     * @returns {Object}
     */
    init: function(obj) {
      var conf = []
        , hosts = {};

      if (!obj.global.server && obj.hosts.length >= 1) {
        obj.hosts.forEach(function(value) {
          for (var key in value) {

            if (key === 'server' || obj.global.server) {
              hosts.server = config.server(obj.global, value);
            }

            if (key === 'port' || obj.global.port) {
              hosts.port = config.port(obj.global, value);
            }

            if (key === 'username' || obj.global.username) {
              hosts.username = config.username(obj.global, value);
            }

            if (key === 'password' || obj.global.password) {
              hosts.password = config.password(obj.global, value);
            }

            if (key === 'privateKey' || obj.global.privateKey) {
              hosts.privateKey = config.privateKey(obj.global, value);
            }

            if (key === 'logs') {
              hosts.logs = config.logs(obj.global, value);
            }

            if (key === 'ignore') {
              hosts.ignore = config.ignore(obj.global, value);
            }

            conf.push(hosts);
          }
        });
      }

      return conf;
    },

    /**
     * @function server
     * @scope private
     * @abstract Setup host server name/ip
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object key name
     *
     * @returns {String}
     */
    server: function(obj, defined) {
      return (defined.server) ?
        defined.server :
          ((obj.server) ? obj.server : false);
    },

    /**
     * @function port
     * @scope private
     * @abstract Setup host port
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {Integer}
     */
    port: function(obj, defined) {
      return (defined.port) ?
        defined.port :
          ((obj.port) ? obj.port : 22);
    },

    /**
     * @function username
     * @scope private
     * @abstract Setup host username
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {String}
     */
    username: function(obj, defined) {
      return (defined.username) ?
        defined.username :
          ((obj.username) ? obj.username : false);
    },

    /**
     * @function password
     * @scope private
     * @abstract Setup host password
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {String}
     */
    password: function(obj, defined) {
      return (defined.password) ?
        defined.password :
          ((obj.password) ? obj.password : false);
    },

    /**
     * @function privateKey
     * @scope private
     * @abstract Setup host privateKey
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {Buffer}
     */
    privateKey: function(obj, defined) {
      return fs.readFileSync((defined.privateKey) ?
        defined.privateKey :
          ((obj.privateKey) ?
            obj.global.privateKey : ''));
    },

    /**
     * @function logs
     * @scope private
     * @abstract Setup host logs
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {Array}
     */
    logs: function(obj, defined) {
      var logs = [];

      if (defined.logs.length >= 1) {
        defined.logs.forEach(function(log) {
          logs.push(log);
        });
      }

      if (obj.logs.length >= 1) {
        obj.logs.forEach(function(log) {
          logs.push(log);
        });
      }

      return logs;
    },

    /**
     * @function ignore
     * @scope private
     * @abstract Setup host ignore
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {Array}
     */
    ignore: function(obj, defined) {
      var ignore = [];

      if (defined.ignore.length >= 1) {
        defined.ignore.forEach(function(log) {
          ignore.push(log);
        });
      }

      if (obj.ignore.length >= 1) {
        obj.global.ignore.forEach(function(log) {
          ignore.push(log);
        });
      }

      return ignore;
    }
  }

  /**
   * @method libs
   * @scope private
   * @abstract Miscellaneous helper libraries
   */
  var libs = libs || {

    /**
     * @function merge
     * @scope private
     * @abstract Perform preliminary option/default object merge
     *
     * @param {Object} defaults Application defaults
     * @param {Object} obj User supplied object
     *
     * @returns {Object}
     */
    merge: function(defaults, obj) {
      defaults = defaults || {};
      return merge(defaults, obj);
    },

    /**
     * @function tail
     * @scope private
     * @abstract Create executable tail command for host(s)
     *
     * @param {Array} logs Array of log(s) to monitor
     *
     * return {Array}
     */
    tail: function(logs) {
      var ret = [];

      if (logs.length >= 1) {
        logs.forEach(function(log){
          ret.push('tail -f -n0 '+log);
        })
      }

      return ret;
    },

    /**
     * @function functions
     * @scope private
     * @abstract Create functions for handling host(s), popogates results
     *           to libs.stream()
     *
     * @param {Obect} host object with connection param(s)
     *
     * @returns {Array}
     */
    functions: function(hosts) {
      var funcs = []
        , logs = [];

      if (hosts.length >= 1) {
        hosts.forEach(function(host) {

          logs = libs.tail(host.logs);

          funcs.push(function fn(callback) {
            conn.on('ready', function ready() {
              logs.forEach(function tail(log) {
                conn.exec(log, function cb(err, stream) {
                  if (err) callback(err);

                  libs.stream({
                    server: host.server,
                    log: log,
                    stream: stream
                  }, callback);

                });
              });
            }).connect(host);
          });
        });
      }

      return funcs;
    },

    /**
     * @function stream
     * @scope private
     * @abstract Handle return stream events
     *
     * @param {Object} stream SSH2 stream
     * @param {Function} fn Return function
     */
    stream: function(stream, fn) {
      stream.stream.on('exit', function(code, signal) {
        /* setTimeout to reconnect */
      }).on('close', function() {
        /* setTimeout to reconnect */
      }).on('data', function(data) {
        /* send to parser plugin system */
        /* send parsed object to fn() */
        fn(null, {
          server: stream.server,
          log: stream.log.replace('tail \-f \-n0', ''),
          data: data.toString('utf8').split('\n', 1)[0].trim()
        });
      });
    }
  };

  /**
   * @function init
   * @scope public
   * @abstract Constructor
   *
   * @param {Object} options User supplied configuration object
   * @param {Function} fn User supplied function
   */
  var init = function(options, fn) {
    fn = fn || options;

    var opts = libs.merge(defaults, options)
      , funcs = []
      , tails = [];

    opts.hosts = config.init(opts);

    if (opts.hosts.length >= 1) {
      funcs = libs.merge(funcs, libs.functions(opts.hosts));
    } else {
      throw new Exception('You must define at least one host to monitor');
    }

    async.parallelLimit(funcs, opts.global.threshold, function cb(err, stream) {
      if (err) throw err;
      fn(null, stream[0]);
    });

  }(options, fn);

};

exports.proginoskes = proginoskes;

==> proginoskes-0.1.3.tgz: package/lib/proginoskes.js <==
/*!
 * proginoskes
 * Copyright(c) 2014-2015 Jason Gerfen <jason.gerfen@gmail.com>
 * License: MIT
 */

var version = 'v0.1.3'
  , fs = require('fs')
  , os = require('os')
  , ssh = require('ssh2')
  , async = require('async')
  , merge = require('deepmerge')
  , conn = new ssh()
  , proginoskes = function(options, fn) {

  'use strict';

  /**
   * @object defaults
   * @abstract Default set of options
   *
   * @param Object} global - Global object for proginoskes
   * @param {Array} hosts - Array of configured streams
   */
  var defaults = {
    global: {
      threshold: os.cpus().length * 2,
      port: 22,
      logs: [
        '/var/log/ufw.log'
      ],
      ignore: {
        destination: [
          'localhost',
          '127.0.0.1'
        ],
        source: [
          'localhost',
          '127.0.0.1'
        ],
        ports: [
          53
        ]
      }
    },
    hosts: []
  };

  /**
   * @method config
   * @scope private
   * @abstract Configuration object
   */
  var config = config || {

    /**
     * @function init
     * @scope private
     * @abstract Creates new host object(s) from configuration
     *
     * @param {Object} obj Merged options
     *
     * @returns {Object}
     */
    init: function(obj) {
      var conf = []
        , hosts = {};

      if (!obj.global.server && obj.hosts.length >= 1) {
        obj.hosts.forEach(function(value) {
          for (var key in value) {

            if (key === 'server' || obj.global.server) {
              hosts.server = config.server(obj.global, value);
            }

            if (key === 'port' || obj.global.port) {
              hosts.port = config.port(obj.global, value);
            }

            if (key === 'username' || obj.global.username) {
              hosts.username = config.username(obj.global, value);
            }

            if (key === 'password' || obj.global.password) {
              hosts.password = config.password(obj.global, value);
            }

            if (key === 'privateKey' || obj.global.privateKey) {
              hosts.privateKey = config.privateKey(obj.global, value);
            }

            if (key === 'logs') {
              hosts.logs = config.logs(obj.global, value);
            }

            if (key === 'ignore') {
              hosts.ignore = config.ignore(obj.global, value);
            }

            conf.push(hosts);
          }
        });
      }

      return conf;
    },

    /**
     * @function server
     * @scope private
     * @abstract Setup host server name/ip
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object key name
     *
     * @returns {String}
     */
    server: function(obj, defined) {
      return (defined.server) ?
        defined.server :
          ((obj.server) ? obj.server : false);
    },

    /**
     * @function port
     * @scope private
     * @abstract Setup host port
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {Integer}
     */
    port: function(obj, defined) {
      return (defined.port) ?
        defined.port :
          ((obj.port) ? obj.port : 22);
    },

    /**
     * @function username
     * @scope private
     * @abstract Setup host username
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {String}
     */
    username: function(obj, defined) {
      return (defined.username) ?
        defined.username :
          ((obj.username) ? obj.username : false);
    },

    /**
     * @function password
     * @scope private
     * @abstract Setup host password
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {String}
     */
    password: function(obj, defined) {
      return (defined.password) ?
        defined.password :
          ((obj.password) ? obj.password : false);
    },

    /**
     * @function privateKey
     * @scope private
     * @abstract Setup host privateKey
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {Buffer}
     */
    privateKey: function(obj, defined) {
      return fs.readFileSync((defined.privateKey) ?
        defined.privateKey :
          ((obj.privateKey) ? obj.privateKey : ''));
    },

    /**
     * @function logs
     * @scope private
     * @abstract Setup host logs
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {Array}
     */
    logs: function(obj, defined) {
      var logs = [];

      if (defined.logs.length >= 1) {
        defined.logs.forEach(function(log) {
          logs.push(log);
        });
      }

      if (obj.logs.length >= 1) {
        obj.logs.forEach(function(log) {
          logs.push(log);
        });
      }

      return logs;
    },

    /**
     * @function ignore
     * @scope private
     * @abstract Setup host ignore
     *
     * @param {Object} obj Configuration object
     * @param {Object} defined Object
     *
     * @returns {Array}
     */
    ignore: function(obj, defined) {
      var ignore = [];

      if (defined.ignore.length >= 1) {
        defined.ignore.forEach(function(log) {
          ignore.push(log);
        });
      }

      if (obj.ignore.length >= 1) {
        obj.global.ignore.forEach(function(log) {
          ignore.push(log);
        });
      }

      return ignore;
    }
  }

  /**
   * @method libs
   * @scope private
   * @abstract Miscellaneous helper libraries
   */
  var libs = libs || {

    /**
     * @function merge
     * @scope private
     * @abstract Perform preliminary option/default object merge
     *
     * @param {Object} defaults Application defaults
     * @param {Object} obj User supplied object
     *
     * @returns {Object}
     */
    merge: function(defaults, obj) {
      defaults = defaults || {};
      return merge(defaults, obj);
    },

    /**
     * @function tail
     * @scope private
     * @abstract Create executable tail command for host(s)
     *
     * @param {Array} logs Array of log(s) to monitor
     *
     * return {Array}
     */
    tail: function(logs) {
      var ret = [];

      if (logs.length >= 1) {
        logs.forEach(function(log){
          ret.push('tail -f -n0 '+log);
        })
      }

      return ret;
    },

    /**
     * @function functions
     * @scope private
     * @abstract Create functions for handling host(s), popogates results
     *           to libs.stream()
     *
     * @param {Obect} host object with connection param(s)
     *
     * @returns {Array}
     */
    functions: function(hosts) {
      var funcs = []
        , logs = [];

      if (hosts.length >= 1) {
        hosts.forEach(function(host) {

          logs = libs.tail(host.logs);

          funcs.push(function fn(callback) {
            conn.on('ready', function ready() {
              logs.forEach(function tail(log) {
                conn.exec(log, function cb(err, stream) {
                  if (err) callback(err);

                  libs.stream({
                    server: host.server,
                    log: log,
                    stream: stream
                  }, callback);

                });
              });
            }).connect(host);
          });
        });
      }

      return funcs;
    },

    /**
     * @function stream
     * @scope private
     * @abstract Handle return stream events
     *
     * @param {Object} stream SSH2 stream
     * @param {Function} fn Return function
     */
    stream: function(stream, fn) {
      stream.stream.on('exit', function(code, signal) {
        /* setTimeout to reconnect */
      }).on('close', function() {
        /* setTimeout to reconnect */
      }).on('data', function(data) {
        /* send to parser plugin system */
        /* send parsed object to fn() */
        fn(null, {
          server: stream.server,
          log: stream.log.replace('tail \-f \-n0', ''),
          data: data.toString('utf8').split('\n', 1)[0].trim()
        });
      });
    }
  };

  /**
   * @function init
   * @scope public
   * @abstract Constructor
   *
   * @param {Object} options User supplied configuration object
   * @param {Function} fn User supplied function
   */
  var init = function(options, fn) {
    fn = fn || options;

    var opts = libs.merge(defaults, options)
      , funcs = []
      , tails = [];

    opts.hosts = config.init(opts);

    if (opts.hosts.length >= 1) {
      funcs = libs.merge(funcs, libs.functions(opts.hosts));
    } else {
      throw new Exception('You must define at least one host to monitor');
    }

    async.parallelLimit(funcs, opts.global.threshold, function cb(err, stream) {
      if (err) throw err;
      fn(null, stream[0]);
    });

  }(options, fn);

};

exports.proginoskes = proginoskes;
